@@ -62,13 +62,15 @@ Group::Group(AppInfo* appInfo, bool pinned) : mGroupMenu(this)
 	gtk_label_set_use_markup(GTK_LABEL(mLabel), true);
 	gtk_container_add(GTK_CONTAINER(overlay), mImage);
 	gtk_overlay_add_overlay(GTK_OVERLAY(overlay), mLabel);
-	gtk_widget_set_halign(GTK_WIDGET(mLabel), GTK_ALIGN_START);
-	gtk_widget_set_valign(GTK_WIDGET(mLabel), GTK_ALIGN_START);
+	gtk_widget_set_halign(mLabel, GTK_ALIGN_START);
+	gtk_widget_set_valign(mLabel, GTK_ALIGN_START);
 	gtk_overlay_set_overlay_pass_through(GTK_OVERLAY(overlay), mLabel, true);
 	gtk_container_add(GTK_CONTAINER(mButton), overlay);
 
 	Help::Gtk::cssClassAdd(mButton, "flat");
 	Help::Gtk::cssClassAdd(mButton, "group");
+	Help::Gtk::cssClassAdd(mLabel, "window_count");
+
 	g_object_set_data(G_OBJECT(mButton), "group", this);
 	gtk_button_set_relief(GTK_BUTTON(mButton), GTK_RELIEF_NONE);
 	gtk_drag_dest_set(mButton, GTK_DEST_DEFAULT_DROP, entries, 1, GDK_ACTION_MOVE);
@@ -150,7 +152,7 @@ Group::Group(AppInfo* appInfo, bool pinned) : mGroupMenu(this)
 					w->mGroupMenuItem->mPreviewTimeout.start();
 				});
 
-			return true;
+			return false;
 		}),
 		this);
 
@@ -208,10 +210,10 @@ void Group::add(GroupWindow* window)
 	mWindows.push(window);
 	mWindowsCount.updateState();
 	mGroupMenu.add(window->mGroupMenuItem);
-	Help::Gtk::cssClassAdd(GTK_WIDGET(mButton), "open_group");
+	Help::Gtk::cssClassAdd(mButton, "open_group");
 
 	if (mWindowsCount == 1 && !mPinned)
-		gtk_box_reorder_child(GTK_BOX(Dock::mBox), GTK_WIDGET(mButton), -1);
+		gtk_box_reorder_child(GTK_BOX(Dock::mBox), mButton, -1);
 
 	gtk_widget_queue_draw(mButton);
 }
@@ -224,7 +226,7 @@ void Group::remove(GroupWindow* window)
 	mSFocus = false;
 
 	if (!mWindowsCount)
-		Help::Gtk::cssClassRemove(GTK_WIDGET(mButton), "open_group");
+		Help::Gtk::cssClassRemove(mButton, "open_group");
 
 	gtk_widget_queue_draw(mButton);
 }
@@ -290,28 +292,49 @@ void Group::resize()
 
 void Group::onDraw(cairo_t* cr)
 {
-	int w = gtk_widget_get_allocated_width(GTK_WIDGET(mButton));
-	int h = gtk_widget_get_allocated_height(GTK_WIDGET(mButton));
-	
+	int w = gtk_widget_get_allocated_width(mButton);
+	int h = gtk_widget_get_allocated_height(mButton);
+
 	double rgba[4];
-	if (mSFocus)
+
+	if (Settings::indicatorColorFromTheme)
 	{
-		rgba[0] = (*Settings::indicatorColor).red;
-		rgba[1] = (*Settings::indicatorColor).green;
-		rgba[2] = (*Settings::indicatorColor).blue;
-		rgba[3] = (*Settings::indicatorColor).alpha;
+		GtkWidget* menu = gtk_menu_new();
+		GtkStyleContext* sc = gtk_widget_get_style_context(menu);
+
+		GValue gv = G_VALUE_INIT;
+		gtk_style_context_get_property(sc, "color", GTK_STATE_FLAG_NORMAL, &gv);
+		GdkRGBA* indicatorColor = (GdkRGBA*)g_value_get_boxed(&gv);
+
+		gtk_widget_destroy(menu);
+
+		rgba[0] = (*indicatorColor).red;
+		rgba[1] = (*indicatorColor).green;
+		rgba[2] = (*indicatorColor).blue;
+		rgba[3] = (*indicatorColor).alpha;
 	}
 	else
 	{
-		rgba[0] = (*Settings::inactiveColor).red;
-		rgba[1] = (*Settings::inactiveColor).green;
-		rgba[2] = (*Settings::inactiveColor).blue;
-		rgba[3] = (*Settings::inactiveColor).alpha;
+		if (mSFocus)
+		{
+			rgba[0] = (*Settings::indicatorColor).red;
+			rgba[1] = (*Settings::indicatorColor).green;
+			rgba[2] = (*Settings::indicatorColor).blue;
+			rgba[3] = (*Settings::indicatorColor).alpha;
+		}
+		else
+		{
+			rgba[0] = (*Settings::inactiveColor).red;
+			rgba[1] = (*Settings::inactiveColor).green;
+			rgba[2] = (*Settings::inactiveColor).blue;
+			rgba[3] = (*Settings::inactiveColor).alpha;
+		}
 	}
 
 	const float BAR_WEIGHT = 0.935;
 	const double DOT_RADIUS = h * 0.065;
-	
+	const double CIRCLE_WEIGHT = 0.0375;
+
 	int orientation = Settings::indicatorOrientation;
 	// Orientation based on panel mode and position
 	// Mimics Windows 10 style, indicator stays on outside
@@ -336,8 +359,12 @@ void Group::onDraw(cairo_t* cr)
 				orientation = ORIENTATION_BOTTOM;
 		}
 	}
+	
+	int indicator_style = Settings::inactiveIndicatorStyle;
+	if (mSFocus)
+		indicator_style = Settings::indicatorStyle;
 
-	switch (Settings::indicatorStyle)
+	switch (indicator_style)
 	{
 	case STYLE_NONE:
 		break;
@@ -396,6 +423,146 @@ void Group::onDraw(cairo_t* cr)
 		break;
 	}
 
+	case STYLE_CILIORA:
+	{
+		if (mSOpened)
+		{
+		    int offset;
+			cairo_set_source_rgba(cr, rgba[0], rgba[1], rgba[2], rgba[3]);
+			
+			offset = 0;
+			
+			if (mSMany) {
+			    if (orientation == ORIENTATION_BOTTOM || orientation == ORIENTATION_TOP) {
+			        offset = 2 * (round(h * (1 - BAR_WEIGHT)));
+			    } else {
+			        offset = (2 * (round(w * (1 - BAR_WEIGHT))));
+			    }
+			}
+
+			if (orientation == ORIENTATION_BOTTOM)
+				cairo_rectangle(cr, 0, round(h * BAR_WEIGHT), w - offset, round(h * (1 - BAR_WEIGHT)));
+			else if (orientation == ORIENTATION_RIGHT)
+				cairo_rectangle(cr, round(w * BAR_WEIGHT), 0, round(w * (1 - BAR_WEIGHT)), h - offset);
+			else if (orientation == ORIENTATION_TOP)
+				cairo_rectangle(cr, 0, 0, w - offset, round(h * (1 - BAR_WEIGHT)));
+			else if (orientation == ORIENTATION_LEFT)
+				cairo_rectangle(cr, 0, 0, round(w * (1 - BAR_WEIGHT)), h - offset);
+
+			cairo_fill(cr);
+		}
+
+		if (mSMany && (mSOpened || mSHover))
+		{
+		    int size;
+			cairo_set_source_rgba(cr, rgba[0], rgba[1], rgba[2], rgba[3]);
+			
+			if (orientation == ORIENTATION_BOTTOM || orientation == ORIENTATION_TOP) {
+			    size = round(h * (1 - BAR_WEIGHT));
+			} else {
+			    size = round(w * (1 - BAR_WEIGHT));
+			}
+			
+			
+			if (orientation == ORIENTATION_BOTTOM)
+				cairo_rectangle(cr, w - size, round(h * BAR_WEIGHT), size, size);
+			else if (orientation == ORIENTATION_RIGHT)
+				cairo_rectangle(cr, round(w * BAR_WEIGHT), h - size, size, size);
+			else if (orientation == ORIENTATION_TOP)
+				cairo_rectangle(cr, w - size, 0, size, size);
+			else if (orientation == ORIENTATION_LEFT)
+				cairo_rectangle(cr, 0, h - size, size, size);
+
+			cairo_fill(cr);
+		}
+		break;
+	}
+
+	case STYLE_CIRCLES:
+	{
+		if (mSOpened)
+		{
+			if (mSMany)
+			{
+				double x0, y0, x1, y1, radius;
+
+				if (orientation == ORIENTATION_BOTTOM)
+				{
+				    radius = h * CIRCLE_WEIGHT;
+					x0 = (w / 2.) - radius * 1.5;
+					x1 = (w / 2.) + radius * 1.5;
+					y0 = y1 = h - radius;
+				}
+				else if (orientation == ORIENTATION_RIGHT)
+				{
+				    radius = w * CIRCLE_WEIGHT;
+					y0 = (h / 2.) - radius * 1.5;
+					y1 = (h / 2.) + radius * 1.5;
+					x0 = x1 = w - radius;
+				}
+				else if (orientation == ORIENTATION_TOP)
+				{
+					radius = h * CIRCLE_WEIGHT;
+					x0 = (w / 2.) - radius * 1.5;
+					x1 = (w / 2.) + radius * 1.5;
+					y0 = y1 = radius;
+				}
+				else if (orientation == ORIENTATION_LEFT)
+				{
+					radius = w * CIRCLE_WEIGHT;
+					y0 = (h / 2.) - radius * 1.5;
+					y1 = (h / 2.) + radius * 1.5;
+					x0 = x1 = radius;
+				}
+
+				cairo_set_source_rgba(cr, rgba[0], rgba[1], rgba[2], rgba[3]);
+
+				cairo_arc(cr, x0, y0, radius, 0.0, 2.0 * M_PI);
+				cairo_fill(cr);
+
+				cairo_set_source_rgba(cr, rgba[0], rgba[1], rgba[2], rgba[3]);
+
+				cairo_arc(cr, x1, y1, radius, 0.0, 2.0 * M_PI);
+				cairo_fill(cr);
+			}
+			else
+			{
+				double x, y, radius;
+
+				if (orientation == ORIENTATION_BOTTOM)
+				{
+					radius = h * CIRCLE_WEIGHT;
+					x = (w / 2.);
+					y = h - radius;
+				}
+				else if (orientation == ORIENTATION_RIGHT)
+				{
+					radius = w * CIRCLE_WEIGHT;
+					x = w - radius;
+					y = (h / 2.);
+				}
+				else if (orientation == ORIENTATION_TOP)
+				{
+					radius = h * CIRCLE_WEIGHT;
+					x = (w / 2.);
+					y = radius;
+				}
+				else if (orientation == ORIENTATION_LEFT)
+				{
+					radius = w * CIRCLE_WEIGHT;
+					x = radius;
+					y = (h / 2.);
+				}
+
+				cairo_set_source_rgba(cr, rgba[0], rgba[1], rgba[2], rgba[3]);
+
+				cairo_arc(cr, x, y, radius, 0.0, 2.0 * M_PI);
+				cairo_fill(cr);
+			}
+		}
+		break;
+	}
+
 	case STYLE_DOTS:
 	{
 		if (mSOpened)
@@ -640,7 +807,7 @@ void Group::onWindowActivate(GroupWindow* groupWindow)
 		mActive = true;
 		mSFocus = true;
 		setTopWindow(groupWindow);
-		Help::Gtk::cssClassAdd(GTK_WIDGET(mButton), "active_group");
+		Help::Gtk::cssClassAdd(mButton, "active_group");
 	}
 }
 
@@ -648,7 +815,7 @@ void Group::onWindowUnactivate()
 {
 	mActive = false;
 	mSFocus = false;
-	Help::Gtk::cssClassRemove(GTK_WIDGET(mButton), "active_group");
+	Help::Gtk::cssClassRemove(mButton, "active_group");
 }
 
 void Group::setTopWindow(GroupWindow* groupWindow)
@@ -670,8 +837,8 @@ void Group::onButtonPress(GdkEventButton* event)
 		GtkWidget* menu = Wnck::buildActionMenu(win, this);
 
 		xfce_panel_plugin_register_menu(Plugin::mXfPlugin, GTK_MENU(menu));
-		gtk_menu_attach_to_widget(GTK_MENU(menu), GTK_WIDGET(mButton), NULL);
-		gtk_menu_popup_at_widget(GTK_MENU(menu), GTK_WIDGET(mButton), GDK_GRAVITY_SOUTH_WEST, GDK_GRAVITY_NORTH_WEST, (GdkEvent*)event);
+		gtk_menu_attach_to_widget(GTK_MENU(menu), mButton, NULL);
+		gtk_menu_popup_at_widget(GTK_MENU(menu), mButton, GDK_GRAVITY_SOUTH_WEST, GDK_GRAVITY_NORTH_WEST, (GdkEvent*)event);
 
 		mGroupMenu.hide();
 	}
@@ -680,7 +847,19 @@ void Group::onButtonPress(GdkEventButton* event)
 void Group::onButtonRelease(GdkEventButton* event)
 {
 	if (event->button == GDK_BUTTON_MIDDLE)
-		closeAll();
+	{
+		switch(Settings::middleButtonBehavior)
+		{
+			case BEHAVIOR_CLOSE_ALL:
+			{
+				closeAll();
+			}
+			case BEHAVIOR_LAUNCH_NEW:
+			{
+				mAppInfo->launch();
+			}
+		}
+	}
 	else if (event->state & GDK_SHIFT_MASK || (mPinned && !mWindowsCount))
 		mAppInfo->launch();
 	else if (mActive)
